#include <stdio.h>
#include <math.h>
/* file_trig.c : file sorgente di un programma che crea NCALC files
                 ciascuno dei quali ha un nome diverso dagli altri (in
                 verita' i nomi si differenziano solo per il carattere
                 al posto del "?"  nella scritta "sencos?.dat"). In
                 ciascuno di questi files vengono scritti i valori di
                 un angolo, del seno e del coseno corrispondenti
                 all'angolo stesso. Successivamente, ognuno dei files
                 viene riaperto e viene letto il suo contenuto, quindi
                 si effettua il confronto tra il rapporto seno/coseno
                 dell'angolo e il valore di tan(angolo) dove tan e' la
                 funzione della libreria matematica che calcola la
                 tangente. Le differenze tra le due quantita' cosi'
                 calcolate vengono riportate in un'altro file il cui
                 nome e' "file_trig.out". */
/* ### */
/* La prossima definizione consente di cambiare facilmente il numero
   totale di files prodotti con all'interno i valori di un angolo, del
   seno e del coseno corrispondenti all'angolo stesso.
   Si consiglia di non utilizzare un numero di NCALC > 10 altrimenti
   potrebbero apparire dei caratteri sgraditi al posto del "?"  nei
   nomi dei files del tipo "sencos?.dat" */
#define NCALC 10
/* ### */
int main()
{
  int i;
  double pigreco, angolo, seno, coseno, tangente, err, errmax;
  char nomefile[12];
  FILE *ifp, *ofp;
  /* Definizione di pigreco (di solito lo si pone uguale a 4 volte il
     valore dell'arcotangente di 1, calcolata utilizzando la funzione
     atan che fa parte della libreria matematica) */
  pigreco = 4. * atan(1.);
  /* Poniamo la stringa nomefile uguale a "sencos?.dat" utilizzando
     la funzione sprintf */
  sprintf(nomefile,"sencos?.dat");
  /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore i
     va da 0 a NCALC-1 */
  for(i=0; i<NCALC; i++) {
    /* Se i e' uguale a 0 poniamo angolo uguale a 0. ,
       se i e' uguale a 1 poniamo angolo uguale a pigreco / (2. * NCALC) ,
       ...
       se i e' uguale a NCALC-1 poniamo angolo uguale a
       (pigreco * (NCALC-1)) / (2. * NCALC) .
       (Attenzione! E' bene evitare la divisione tra interi) */
    angolo = ( 0.5 * i ) / NCALC * pigreco;
    /* Poniamo la variabile seno uguale al valore del seno di angolo */
    seno = sin(angolo);
    /* Poniamo la variabile coseno uguale al valore del coseno di angolo */
    coseno = cos(angolo);
    /* Nella stringa nomefile sostituiamo al carattere "?" uno "0" se
       i e' uguale a 0, un "1" se i e' uguale a 1, etc.
       (Consiglio 1: si cerchi l'indice dell'elemento del vettore
        nomefile che corrisponde al carattere "?";
	Consiglio 2: si ricordi che il codice ASCII corrispondente al
	carattere "0" e' 48 (o 060 in ottale), quello corrispondente
	a "1" e' 49, etc. */
    nomefile[6] = 060 + i;
    /* Apriamo in modalita' di scrittura il file di output che si
       chiama nomefile, associamo a ofp l'indirizzo corrispondente al
       file */
    ofp = fopen(nomefile, "w");
    /* Scriviamo i valori delle variabili angolo, seno e coseno
       sul file che si chiama nomefile.
       (Consiglio 1: e' meglio scrivere i valori delle variabili in
        modo binario, utilizzando la funzione fwrite. In tal modo
	siamo sicuri di non perdere informazioni durante la
	conversione in ASCII nell'operazione di scrittura e nella
	conseguente lettura.
	Consiglio 2: se fwrite risulta proprio indigesta, si puo'
	comunque utilizzare fprintf ...) */
    fwrite(&angolo,sizeof(double),1,ofp);
    fwrite(&seno,sizeof(double),1,ofp);
    fwrite(&coseno,sizeof(double),1,ofp);
    /* Chiudiamo il file che si chiama nomefile */
    fclose(ofp);
  }
  /* Poniamo il valore di errmax uguale a 0 */
  errmax = 0.;
  /* Apriamo in modalita' di scrittura il file di output che si chiama
     "file_trig.out", associamo a ofp l'indirizzo corrispondente al
     file */
  ofp = fopen("file_trig.out", "w");
  /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore i
     va da 0 a NCALC-1 */
  for(i=0; i<NCALC; i++) {
    /* Nella stringa nomefile sostituiamo al carattere "?" uno "0" se
       i e' uguale a 0, un "1" se i e' uguale a 1, etc.
       (Consiglio: si puo' fare esattamente come poche righe sopra) */
    nomefile[6] = 060 + i;
    /* Apriamo in modalita' di lettura il file di output che si chiama
       nomefile, associamo a ifp l'indirizzo corrispondente al file */
    ifp = fopen(nomefile, "r");
    /* Leggiamo i valori delle variabili angolo, seno e coseno
       dal file che si chiama nomefile.
       (Consiglio: se prima si e' utilizzato fwrite, adesso e' il
        momento di fread, se invece si e' utilizzato fprintf si puo'
        lavorare con fscanf) */
    fread(&angolo,sizeof(double),1,ifp);
    fread(&seno,sizeof(double),1,ifp);
    fread(&coseno,sizeof(double),1,ifp);
    /* Chiudiamo il file che si chiama nomefile */
    fclose(ifp);
    /* Poniamo la variabile tangente uguale al valore della tangente
       di angolo */
    tangente = tan(angolo);
    /* Poniamo la variabile err uguale al valore assoluto della
       differenza tra tangente e seno / coseno */
    err = fabs( tangente - seno / coseno);
    /* Scriviamo i valori delle variabili angolo, seno e coseno
       sul file che si chiama "file_trig.out".
       (Consiglio: adesso si scrivano utilizzando fprintf, cosi'
        alla fine dell'esecuzione del programma potremo andare
        a controllare l'esito di ciascuna operazione guardando
        il file "file_trig.out") */
    fprintf(ofp,"%lf %le\n", angolo, err);
    /* La variabile di errmax deve contenere il valore massimo
       assunto da err, quindi se err > errmax allora ... */
    if(errmax < err)
      errmax = err;
  }
  /* Chiudiamo il file che si chiama "file_trig.out" */
  fclose(ofp);
  /* Stampiamo su video il valore del massimo errore che e' contenuto
     nella variabile errmax, se tutto e' andato bene tale valore deve
     essere molto piccolo */
  printf("Errore massimo = %le\n", errmax);
  exit(0);
}
