#include <stdio.h>
/* ordina_elenco.c : file sorgente di un programma che legge l'elenco
                     degli studenti (con il loro login name e il
                     gruppo di appartenenza) dal file "belelenco.dat".
                     Dopo aver ordinato l'elenco secondo un criterio
                     che puo' essere scelto cambiando poche istruzioni
                     nel programma (ad es. l'elenco puo' essere
                     riscritto seguendo l'ordine alfabetico di cognome
                     e nome, oppure del login name, etc.), l'elenco
                     viene riscritto nel file "ordelenco.dat". */
/* ### */
/* La prossima definizione consente di cambiare facilmente la
   lunghezza massima consentita dei nomi e dei cognomi */
#define LUNGMAX 25
/* La prossima definizione consente di cambiare facilmente il
   numero massimo degli studenti */
#define NMAXSTUD 100
/* Definizione dei prototipi delle funzioni */
void incastra_ordinato(char elenco[][], int inizio, int separa,
		       int fine, int iniz_chiave, int lung_chiave);
void ordina(char elenco[][], int nstud, int iniz_chiave, int lung_chiave);
void scambia_righe(char elenco[][], int riga1, int riga2);
/* ### */
int main()
{
  int i, nstud;
  int iniz_chiave, lung_chiave;
  char intestazione[3*LUNGMAX];
  char elenco[NMAXSTUD][3*LUNGMAX];
  FILE *ifp, *ofp;
  /* Apriamo il file di input "belelenco.dat" in lettura,
     associamo a ifp l'indirizzo corrispondente al file */
  ifp = fopen("belelenco.dat", "r");
  /* Leggiamo la prima riga di intestazione in cima al file di input */
  fgets(intestazione, 3*LUNGMAX, ifp);
  /* Inizializziamo la variabile nstud (che rappresenta il numero di
     studenti) uguale a 0 */ 
  nstud = 0;
  /* Leggiamo una riga del file di input e la scriviamo nella
     nstud-esima riga della matrice elenco utilizzando
     l'istruzione fgets.
     Questa istruzione di lettura la mettiamo all'interno di una
     condizione di esecuzione di un ciclo while(...) {...}.
     Continuiamo ad eseguire il ciclo, mentre e' verificata la
     condizione che il risultato restituito da fgets sia diverso da
     NULL */
  while( fgets(&elenco[nstud][0], 3*LUNGMAX, ifp) != NULL) {
    /* Incrementiamo di 1 la variabile nstud */
    nstud++;
  }
  /* Chiudiamo il file di input */
  fclose(ifp);
  /* Vogliamo mettere in fila l'elenco degli studenti secondo l'ordine
     alfabetico dei loro cognomi e dei loro nomi: sappiamo che
     (1) ci sono 10 caratteri prima che inizi il cognome
     (2) cognome e nome insieme sono lunghi al piu' 2*LUNGMAX-2 caratteri */
  /* Allora poniamo iniz_chiave uguale a 10 ... */
  iniz_chiave = 10;
  /* ... e lung_chiave uguale a 2*LUNGMAX-2 caratteri */
  lung_chiave = 2 * LUNGMAX - 2;
  /* Ordiniamo l'elenco */
  ordina(elenco, nstud, iniz_chiave, lung_chiave);
  /* Apriamo il file di output "ordelenco.dat" in scrittura,
     associamo a ofp l'indirizzo corrispondente al file */
  ofp = fopen("ordelenco.dat", "w");
  /* Scriviamo la riga di intestazione nel file di output */
  fprintf(ofp, intestazione);
  /* Eseguiamo un ciclo sulla variabile intera chi i in modo
     da considerare tutti gli nstud studenti che fanno parte
     dell'elenco */
  for(i=0; i<nstud; i++) {
    /* Scriviamo sul file di output la i-esima riga della matrice
       elenco */
    fprintf(ofp,"%s", &elenco[i][0]);
  }
  /* Chiudiamo il file di output */
  fclose(ofp);
  exit(0);
}
/* ### */
void incastra_ordinato(elenco, inizio, separa, fine, iniz_chiave, lung_chiave)
     char elenco[][3*LUNGMAX];
     int inizio, separa, fine, iniz_chiave, lung_chiave;
{
  int i, i1, i2, j, vetord[NMAXSTUD];
  /* Inizializziamo il contatore i1 uguale a inizio ... */
  i1 = inizio;
  /* ... e il contaatore i2 uguale a separa ... */
  i2 = separa;
  /* ... e il contatore i uguale a inizio */
  i = inizio;
  /* A questo punto vogliamo definire il vettore vetord in modo che
     contenga gli indici delle righe (comprese tra la inizio-esima e
     la fine-1-esima) secondo il criterio di ordinamento che viene
     considerato.
     A questo scopo eseguiamo un ciclo di istruzioni mentre sono
     verificate ENTRAMBE le due seguenti condizioni:
     (1) il contatore i1 deve essere minore di separa,
     (2) il contatore i2 deve essere minore di fine */
  while( ( i1 < separa ) && ( i2 < fine ) ) {
    /* Eseguiamo il confronto tra la i1-esima riga di elenco
       e la i2-esima riga limitando il confronto a partire
       dal carattere iniz_chiave-esimo per un numero di
       caratteri uguale a lung_chiave.
       Se (limitatamente ai caratteri confrontati) la i1-esima
       riga viene prima della i2-esima o sono uguali ... */
    if( strncmp( &elenco[i1][iniz_chiave] ,
		 &elenco[i2][iniz_chiave] ,  lung_chiave ) <= 0 ) {
      /* ... allora poniamo l'i-esimo elemento del vettore vetord
	 uguale ad i1 */
      vetord[i] = i1;
      /* Incrementiamo di 1 il valore del contatore i1 */
      i1++;
    }
    /* Altrimenti ... */
    else {
      /* ... poniamo l'i-esimo elemento del vettore vetord
	 uguale ad i2 */
      vetord[i] = i2;
      /* Incrementiamo di 1 il valore del contatore i2 */
      i2++;
    }
    /* Incrementiamo di 1 il valore del contatore i */
    i++;
  }
  /* Attenzione!!! Mancano le definizioni di alcuni elementi
     del vettore vetord!!!
     Infatti, mancano separa-1-i1 definizioni (se siamo usciti dal
     ciclo precedente perche' i2 = fine) oppure mancano fine-1-i2
     definizioni (se siamo usciti dal ciclo precedente perche' i1 =
     separa). */
  /* Di conseguenza, eseguiamo un ciclo di istruzioni sul contatore i1
     che parte dal valore che esso ha attualmente fino a separa-1 */
  for(;i1<separa;i1++) {
    /* Poniamo l'i-esimo elemento del vettore vetord uguale ad i1 */
    vetord[i] = i1;
    /* Incrementiamo di 1 il valore del contatore i */
    i++;
  }
  /* Adesso eseguiamo un ciclo di istruzioni sul contatore i2
     che parte dal valore che esso ha attualmente fino a fine-1 */
  for(;i2<fine;i2++) {
    /* Poniamo l'i-esimo elemento del vettore vetord uguale ad i2 */
    vetord[i] = i2;
    /* Incrementiamo di 1 il valore del contatore i */
    i++;
  }
  /* A questo punto eseguiamo effettivamente lo scambio delle righe
     dell'elenco (comprese tra la inizio-esima e la fine-1-esima)
     secondo il criterio di ordinamento che viene considerato.
     A questo scopo eseguiamo un ciclo di istruzioni mentre il
     contatore i va da inizio a fine-1 */
  for(i=inizio;i<fine;i++) {
    /* Eseguiamo le istruzioni seguenti solo se e' verificata
       la condizione che i sia diverso da vetord[i] (altrimenti
       non c'e' bisogno di fare alcunche') */
    if(i != vetord[i]) {
      /* Eseguiamo lo scambio tra la i-esima riga e la
	 vetord[i]-iesima riga dell'elenco facendo appello alla
	 routine scambia_righe */
      scambia_righe(elenco, i, vetord[i]);
      /* Attenzione!!! Siccome abbiamo cambiato di posto un paio
	 di righe dobbiamo aggiornare il vettore vetord!!! */
      /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore
	 j va da i+1 a fine-1 */
      for(j=i+1; j<fine; j++) {
	/* Se il j-esimo elemento di vetord e' proprio uguale ad i ... */
	if(vetord[j] == i) {
	  /* ... poniamo il j-esimo elemento di vetord uguale
	     all'i-esimo elemento di vetord ... */
	  vetord[j] = vetord[i];
	  /* ... e, con un'istruzione break, usciamo dal ciclo che
	     utilizza j come contatore */
	  break;
	}
      }
      /* Poniamo l'i-esimo elemento di vetord uguale a i */
      vetord[i] = i;
    }
  }
}
/* ### */
void ordina(elenco, nstud, iniz_chiave, lung_chiave)
     char elenco[][3*LUNGMAX];
     int nstud, iniz_chiave, lung_chiave;
{
  int pot2, inizio, separa, fine;
  /* Inizializziamo la variabile pot2 uguale ad 1 */
  pot2 = 1;
  /* Continuiamo ad eseguire il seguente ciclo di istruzioni mentre e'
     verificata l'ipotesi che pot2 sia minore di nstud */
  while(pot2 < nstud) {
    /* Inizializziamo la variabile inizio uguale a 0 ... */
    inizio = 0;
    /* ... e la variabile separa uguale a pot2 */
    separa = pot2;
    /* Le righe che sono comprese tra la inizio-esima e la
       separa-1-esima sono da considerarsi gia' ordinate dalle
       operazioni precedentemente effettuate durante l'esecuzione
       dell'algoritmo;
       quindi, continuiamo ad eseguire un ciclo di istruzioni mentre
       e' verificata l'ipotesi che separa sia minore di nstud */
    while( separa < nstud ) {
      /* Poniamo la variabile fine uguale a separa+pot2 */
      fine = separa + pot2;
      /* Non ha senso cercare di ordinare le righe successive
	 a nstud (perche' l'elenco degli studenti e' costituito
	 da al piu' nstud righe);
	 quindi se fine e' maggiore di nstud la poniamo uguale
	 ad nstud */
      if(fine > nstud)
	fine = nstud;
      /* A questo punto dobbiamo ordinare il blocco di righe
	 dell'elenco compreso tra la inizio-esima riga e la
	 fine-1-esima, sapendo che i due sottoblocchi compresi (nel
	 primo caso) tra la inizio-esima riga e la separa-1-esima e
	 (nel secondo caso) tra la separa-esima riga e la fine-1-esima
	 sono da considerarsi gia' ordinati.
         Eseguiamo l'ordinamento dell'unione dei due sottoblocchi
	 facendo appello alla routine incastra_ordinato */
      incastra_ordinato(elenco, inizio, separa, fine,
			iniz_chiave, lung_chiave);
      /* Ridefiniamo la variabile inizio uguale a fine ... */
      inizio = fine;
      /*  ... e la variabile separa uguale a inizio + pot2 */
      separa = inizio + pot2;
    }
    /* Raddoppiamo il valore della variabile pot2 */
    pot2 *= 2;
  }
}
/* ### */
void scambia_righe(elenco, riga1, riga2)
     char elenco[][3*LUNGMAX];
     int riga1, riga2;
{
  int i;
  char tmp;
  /* Eseguiamo il seguente ciclo di istruzioni mentre il contatore
     i va da 0 a 3*LUNGMAX (cioe' consideriamo tutte le colonne
     della matrice elenco) */
  for(i=0;i<3*LUNGMAX;i++) {
    /* Poniamo tmp uguale all'i-esimo elemento di colonna della
       riga1-esima riga della matrice elenco */
    tmp = elenco[riga1][i];
    /* Poniamo l'i-esimo elemento di colonna della riga1-esima riga
       della matrice elenco uguale all'i-esimo elemento di colonna
       della riga2-esima riga della stessa matrice elenco */
    elenco[riga1][i] = elenco[riga2][i];
    /* Poniamo l'i-esimo elemento di colonna della riga2-esima riga
       della matrice elenco uguale a tmp */
    elenco[riga2][i] = tmp;
  }
}
