#include <stdio.h>
#include <math.h>
/* radqua.c : file sorgente di un programma che effettua il calcolo
              della radice quadrata di un numero compreso tra 1 e 100
              con un procedimento iterativo che migliora via via
              l'approssimazione del risultato.  L'algoritmo di calcolo
              (ovviamente) fa uso dell'espansione in serie di potenze
              (arrestata al primo ordine) della funzione sqrt(1+t) con
              t prossimo a zero. */
/* ### */
/* La prossima definizione consente di cambiare facilmente il tipo di
   variabili con cui si effettua il calcolo (da double a float o il
   viceversa oppure da double a long double) */
#define DOUBLE double
/* Attenzione!
   Se si vuole utilizzare il tipo float anziche' il tipo double bisogna
   togliere le l dai formati di conversione (cioe' occorre utilizzare
   f o e anziche' lf o le, rispettivamente).
   Se si vuole utilizzare il tipo long double anziche' il tipo double
   bisogna sostituire le l con delle L nei formati di conversione (cioe'
   occorre utilizzare Lf o Le anziche' lf o le, rispettivamente). */
/* ### */
int main()
{
  /* Definizione delle variabili:
     j = contatore
     x = numero del quale vogliamo calcolare la radice quadrata
     y = numero che rappresenta l'approssimazione calcolata della radice
         quadrata di x e che viene migliorata ad ogni iterazione del metodo
     t = numero da cui si calcola facilmente la correzione da
         un'approssimazione a quella successiva
     errrel = errore relativo massimo di tolleranza che viene ammesso
              nel calcolo della radice quadrata */
  int j;
  DOUBLE x, y, t, errrel;
  /* Inizio delle istruzioni eseguibili */
  /* Input di x */
  do {
    printf("Numero x (1 <= x < 100) di cui vuoi ");
    printf("calcolare la radice quadrata?\n");
    scanf("%lf", &x);
  } while (x < 1. || x >= 100.);
  /* Input di errrel */
  do {
    printf("Errore relativo massimo (>0) nel ");
    printf("calcolo della radice quadrata?\n");
    scanf("%lf", &errrel);
  } while (errrel <= 0.);
  /* Fine delle istruzioni di input */
  /* Definisco come approssimazione iniziale y della radice quadrata
     il numero intero massimo tale che il suo quadrato e' minore o uguale
     a x */
  for(j=2;j<=10;j++) {
    if(x < j*j) {
      y = j - 1.;
      break; // Questa istruzione induce l'uscita dal ciclo for che la contiene
    }
  }
  /* Miglioro l'approssimazione della radice quadrata y del numero x,
     finche' la correzione sul valore di y non e' piu' piccola di errrel */
  do {
    t = 0.5 * ( x / (y * y) - 1. );
    /* Scrivo t che e' un'approssimazione dell'errore relativo attuale */
    printf("  Valore temporaneo di (x - y * y) / (y * y) : %10.3le\n",t);
    /* Correggo l'approssimazione della radice quadrata */
    y *= (1. + t);
    /* Eseguo il test di uscita dal ciclo: se il valore assoluto di t e'
       maggiore dell'errore relativo massimo di tolleranza, continuo
       ad iterare le correzioni del calcolo della radice quadrata */
  } while(fabs(t) >= errrel);
  /* Scrivo la radice quadrata del numero x di partenza */
  printf("Il valore y calcolato della radice quadrata di x e' : ");
  printf("%17.16lf\n", y);
  /* Scrivo l'errore relativo finale */
  printf("Il valore assoluto di (x - y * y) / x e' : ");
  printf("%10.3le\n", fabs( (x - y * y) / x ) );
  /* Conclusione delle istruzioni eseguibili */
  exit(0);
}
