#include <stdio.h>
/* trig_ric.c : file sorgente di un programma che calcola in modo
                ricorsivo l'espansione di cos(nx) e sen(nx) in
                funzione delle potenze di cos x e sen x */
/* ### */
/* La prossima definizione consente di cambiare facilmente il
   coefficiente intero n massimo delle funzioni cos(nx) e sen(nx) di
   cui vogliamo calcolare gli sviluppi */
#define NMAX 25
/* Definizione dei prototipi delle funzioni */
void cosn(int svil[], int n);
void senn(int svil[], int n);
void stampasvil(int svil[], int n);
/* ### */
int main()
{
  /* Definizione delle variabili:
     n = coefficiente intero di cos(nx) e sen(nx)
     svil = vettore di coefficienti interi che rappresenta
            lo sviluppo delle funzioni cos(nx) e sen(nx) */
  int n, svil[NMAX+1];
  /* Fase di input del coefficiente intero n */
  do {
    printf("Dammi un numero positivo (<=%d): ", NMAX);
    scanf("%d", &n);
  } while (n <= 0  ||  n > NMAX);
  /* Nel vettore svil riportiamo il calcolo dello sviluppo di
     cos(nx) */
  cosn(svil, n);
  /* Stampiamo "cos nx =  " */
  printf("cos %dx =", n);
  /* Stampiamo lo sviluppo descritto dal vettore svil */
  stampasvil(svil, n);
  /* Nel vettore svil riportiamo il calcolo dello sviluppo di
     sen(nx) */
  senn(svil, n);
  /* Stampiamo "sen nx =  " */
  printf("sen %dx =", n);
  /* Stampiamo lo sviluppo descritto dal vettore svil */
  stampasvil(svil, n);
  exit(0);
}
/* ### */
void cosn(svil, n)
     int svil[];
     int n;
    /* Nota: questa function e' stata scritta in modo tale che
             il calcolo dello sviluppo di cos(nx) sia ricorsivo */
{
  /* Definizione di un contatore i */
  int i;
  /* Definizione di svilnmen1 che e' un vettore (la cui area di
     memoria e' allocata in modo dinamico) di n coefficienti interi che
     rappresenta lo sviluppo delle funzioni cos((n-1)x) o
     sen((n-1)x) */
  int *svilnmen1;
  /* Test di fine ricorsione: n e' uguale a 1? */
  if(n == 1) {
    /* Quando n=1, dobbiamo semplicemente scrivere lo sviluppo di cos(x) */
    svil[0] = 1;
    svil[1] = 0;
  }
  else {
    /* Allocazione della memoria per il vettore svilnmen1 */
    svilnmen1 = malloc(n * sizeof(int));
    /* Nel vettore svilnmen1 riportiamo il calcolo ricorsivo
       dello sviluppo di cos((n-1)x) */
    cosn(svilnmen1, n-1);
    /* Eseguiamo un ciclo sul contatore i che va da 0 a n-1 in modo da
       riportare nel vettore svil il prodotto di cos(x)*cos((n-1)x) */
    for(i=0; i<n; i++) {
      svil[i] = svilnmen1[i];
    }
    /* Azzeriamo l'elemento n-esimo di svil (perche' corrisponde
       al coefficiente di sen^n(x) ) */
    svil[n] = 0;
    /* Nel vettore svilnmen1 riportiamo ora il calcolo ricorsivo
       dello sviluppo di sen((n-1)x) */
    senn(svilnmen1, n-1);
    /* Eseguiamo un ciclo sul contatore i che va da 1 a n in modo da
       aggiungere allo sviluppo precedente nel vettore svil il
       prodotto di -sen(x)*sen((n-1)x) */
    for(i=1; i<=n; i++) {
      svil[i] -= svilnmen1[i-1];
    }
    /* Liberiamo l'area di memoria allocata per il vettore svilnmen1 */
    free(svilnmen1);
  }
  /* Uscita dalla funzione */
  return;
}
/* ### */
void senn(svil, n)
     int svil[];
     int n;
    /* Nota: questa function e' stata scritta in modo tale che
             il calcolo dello sviluppo di sen(nx) sia ricorsivo */
{
  /* Definizione di un contatore i */
  int i;
  /* Definizione di svilnmen1 che e' un vettore (definito
     dinamicamente) di n coefficienti interi che rappresenta lo
     sviluppo delle funzioni cos((n-1)x) o sen((n-1)x) */
  int *svilnmen1;
  /* Test di fine ricorsione: n e' uguale a 1? */
  if(n == 1) {
    /* Quando n=1, dobbiamo semplicemente scrivere lo sviluppo di sen(x) */
    svil[0] = 0;
    svil[1] = 1;
  }
  else {
    /* Allocazione della memoria per il vettore svilnmen1 */
    svilnmen1 = malloc(n * sizeof(int));
    /* Nel vettore svilnmen1 riportiamo il calcolo ricorsivo
       dello sviluppo di cos((n-1)x) */
    cosn(svilnmen1, n-1);
    /* Eseguiamo un ciclo sul contatore i che va da 1 a n in modo da
       riportare nel vettore svil il prodotto di sen(x)*cos((n-1)x) */
    for(i=1; i<=n; i++) {
      svil[i] = svilnmen1[i-1];
    }
    /* Azzeriamo l'elemento 0-esimo di svil (perche' corrisponde
       al coefficiente di cos^n(x) ) */
    svil[0] = 0;
    /* Nel vettore svilnmen1 riportiamo ora il calcolo ricorsivo
       dello sviluppo di sen((n-1)x) */
    senn(svilnmen1, n-1);
    /* Eseguiamo un ciclo sul contatore i che va da 0 a n-1 in modo da
       aggiungere allo sviluppo precedente nel vettore svil il
       prodotto di cos(x)*sen((n-1)x) */
    for(i=0; i<n; i++) {
      svil[i] += svilnmen1[i];
    }
    /* Liberiamo l'area di memoria allocata per il vettore svilnmen1 */
    free(svilnmen1);
  }
  /* Uscita dalla funzione */
  return;
}
/* ### */
void stampasvil(svil, n)
     int svil[];
     int n;
{
  /* Definizione di un contatore i */
  int i;
  /* Definizione di una variabile intera nscritti che conta il numero
     di termini scritti dell'espansione */
  int nscritti;
  /* Azzero nscritti */
  nscritti = 0;
  /* Ciclo su i che va da 0 a n per scrivere tutti i termini dello
     sviluppo */
  for(i=0;i<=n;i++) {
    /* Stampiamo un termine solo se svil[i] e' diverso da zero */
    if(svil[i] != 0) {
      /* Ogni 2 termini scritti dell'espansione stampiamo un ritorno a capo
	 e qualche spazio */
      if(nscritti%2 == 0)  printf("\n   ");
      /* Se nscritti>0 e svil[i]>0 stampiamo il segno + */
      if(nscritti > 0  &&  svil[i] > 0)  printf("+");
      /* Stampiamo l'i-esimo coefficiente dello sviluppo */
      printf("%d ", svil[i]);
      /* Se i<n stampiamo cos^n-i x */
      if(i<n)  printf("cos^%d x ", n-i);
      /* Se i>0 stampiamo sen^i x */
      if(i>0)  printf("sen^%d x ", i);
      /* Se il termine che abbiamo stampato non e' l'ultimo della riga
	 stampiamo uno spazio per separare un termine rispetto
	 all'altro */
      if(nscritti%2 != 1)  printf(" ");
      /* Incrementiamo il numero di termini scritti */
      nscritti++;
    }
  }
  /* Stampa il ritorno a capo */
  printf("\n");  
  return;
}
